/**
* @file Semitic phonetic transliteration utilities
* @version 1.0.0
* @author Greg Borota
* @copyright (c) 2017 Greg Borota.
* @license MIT
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/

// https://peshitta.github.io
// https://sedra.bethmardutho.org/about/fonts
// http://cal1.cn.huc.edu/searching/fullbrowser.html

/** @module phoneticCodeUtil */
/**
 * Semitic 22 base consonants
 * @constant
 * @type { string[] }
*/
var consonants = Object.freeze([
  // abgad
  '\u2019', // Ālaph - Glottal Stop
  'b', // Bēth - Voiced bilabial stop
  'g', // Gāmal - Voiced velar stop
  'd', // Dālath - Voiced alveolar stop

  // hawaz
  'h', // Hē - Voiceless glottal fricative
  'w', // Waw - Voiced labio-velar approximant
  'z', // Zain - Voiced alveolar fricative

  // ḥaṭy
  'ḥ', // Ḥēth - Voiceless pharyngeal fricative
  'ṭ', // Ṭēth - Emphatic pharyngealized T
  'y', // Yodh - Voiced palatal approximant

  // kalman
  'k', // Kāph - Voiceless velar stop
  'l', // Lāmadh - Alveolar lateral approximant
  'm', // Mem - Bilabial nasal
  'n', // Nun - Alveolar nasal

  // saʿpac
  's', // Semkath - Voiceless alveolar fricative
  '\u2018', // ʿĒ - Voiced pharyngeal fricative
  'p', // Pē - Voiceless bilabial stop
  'ṣ', // Ṣādhē - Emphatic pharyngealized s, ~ strike-through for pharyngealization

  // qarshat
  'q', // Qoph - Voiceless uvular stop
  'r', // Rēsh - Alveolar trill
  'š', // Shin - Voiceless palato-alveolar sibilant
  't' // Taw - Voiceless alveolar stop
]);

/**
 * Veghadhkhefath consonants
 * @constant
 * @type { string[] }
 */
var veghadhkhefathConsonants = [
  'ḇ', // Vēth - Voiced labiodental fricative
  'g̱', // Ghāmal - Voiced velar fricative
  'ḏ', // Dhālath - Voiced dental fricative
  'ḵ', // Khāph - Voiceless velar fricative
  'p̱', // Fē - voiceless labiodental fricative
  'ṯ' // // Thaw - Voiceless dental fricative
];

/**
 * Other semitic consonants
 * @constant
 * @type { string[] }
 */
var extraConsonants = [
  'ś', // Śin - Voiceless alveolar lateral fricative
  'ḍ', // Ḍād - Emphatic pharyngealized D, ~ strike-through for pharyngealization
  'ẓ' ];

/**
 * All consonants concatenated
 * @constant
 * @type { string[] }
 */
var allConsonants = Object.freeze(
  consonants.concat(veghadhkhefathConsonants).concat(extraConsonants)
);

/**
 * Common aramaic vowels
 * @constant
 * @type { string[] }
*/
var commonVowels = Object.freeze([
  'a', // Pthāḥā - Open front unrounded vowel
  'ā', // Zqāphā - Open back unrounded vowel
  'ē', // Zlāmā qašyā - Close-mid front unrounded vowel
  'ī', // Ḥḇāṣā - Close front unrounded vowel
  'ū' // Rvāṣā - Close back rounded vowel
]);

/**
 * Eastern/hebrew only vowels
 * @constant
 * @type { string[] }
*/
var easternVowels = Object.freeze([
  'e', // Zlāmā pšīqā - Open-mid front unrounded - short version of e
  'ō' // Rwāḥā - Close-mid back rounded vowel
]);

/**
 * Other semitic vowels
 * @constant
 * @type { string[] }
*/
var otherVowels = Object.freeze([
  'u', // Near-close near-back rounded vowel. Short version of u.
  'i', // Near-close near-front unrounded vowel. Short version of i.
  'o' // Open-mid back rounded vowel. Short version of o.
]);

/**
 * Semitic vowels including Eastern/Hebrew ones and other short vowels.
 * @constant
 * @type { string[] }
*/
var allVowels = Object.freeze(
  commonVowels.concat(easternVowels).concat(otherVowels)
);

/**
 * The silent letter symbol
 * @const
 * @type { string }
 */
var silent = '∅';

/**
 * Is character c a semitic consonant
 * @param { string } c input character
 * @returns { boolean } true if c is semitic consonant
 */
var isConsonant = function (c) { return allConsonants.indexOf(c) > -1; };

/**
 * Is character c a semitic vowel
 * @param { string } c input character
 * @returns { boolean } true if c is semitic vowel
 */
var isVowel = function (c) { return allVowels.indexOf(c) > -1; };

/**
 * Remove vowels, leaving consonantal word only.
 * @param { string } word input word to be processed
 * @returns { string } consonantal word
 */
var removeVowels = function (word) {
  if (!word) {
    return word;
  }

  var hasVowels = false;
  var stack = [];
  for (var i = 0, len = word.length; i < len; i++) {
    var c = word.charAt(i);
    if (isVowel(c)) {
      hasVowels = true;
    } else {
      stack.push(c);
    }
  }
  return hasVowels ? stack.join('') : word;
};

export { consonants, veghadhkhefathConsonants, extraConsonants, allConsonants, commonVowels, easternVowels, otherVowels, allVowels, silent, isConsonant, isVowel, removeVowels };
